C
C  'COMINDX' CONTAINS POINTERS TO BEGINNING AND END OF STARTING
C     LINE (ISL & JSL)  , ABBREVIATED HEADING LINE (IAH & JAH)  ,
C     'MIMIMJMJ' LINE (IMI & JMI)  , END OF REPORT (IEQ)  , AND
C     END OF BULLETIN (IGS)  ,  AND A WORKING POINTERS (IPT)
C     AND (KPT) AND BEGINNING OF THE FIRST LEVEL (LPT).
C     OBS...  IN CASE OF TEMP SHIP REPORT WHEN SHIP'S NAME IS ON
C             SEPARATE LINE POINTERS IMI & JMI POINT TO THE THIS LINE
C
      COMMON / COMINDX / ISL,JSL,IAH,JAH,IMI,JMI,IEQ,IGS,IPT,KPT
     C                  ,LPT,I1
C
