C
C     'COMBUFF' CONTAINS READ IN AREAS USED BY VARIOUS FILES ,
C     AS WELL AS END-OF-FILE INDICATOR.
C
      COMMON / COMBUFF / IEOF,IOPTS(678),IPARAMS(39000),IPOINTS(128)
     C                   ,IMPSTA(2000)
C
C
