C
      COMMON /COMPOIN/ NBPW,NWPT,NBPT,OPS2,
     1                 MASKS(JP14),MBUF(JP3)
C
C     NBPW          -  NUMBER OF BITS PER COMPUTER WORD
C     NWPT          -  POINTER TO WORD IN ARRAY MBUFF
C     NBPT          -  POINTER TO BIT INSIDE AN COMPUTER WORD
C     MASKS         -  ARRAY CONTAINING BIT MASKS
C     MINDIC        -  MISSING DATA INDICATOR
C     MBUF          -  ARRAY CONTAINING SINGLE BUFR MESSAGE
C
