C
C     'COMWORK' CONTAINS WORK AREAS USED BY DECODING ROUTINES AND
C     ALSO THE MISSING DATA INDICATOR.
C
      COMMON / COMWORK / KCHAR(jp15),KINT(jp15),KDEC(jp11),
     C                   KERR,IT1,IT2,MINDIC,KHEAD(jp12),ilen
C
