C Copyright 1981-2016 ECMWF.
C
C This software is licensed under the terms of the Apache Licence 
C Version 2.0 which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
C
C In applying this licence, ECMWF does not waive the privileges and immunities 
C granted to it by virtue of its status as an intergovernmental organisation 
C nor does it submit to any jurisdiction.
C

C
      COMMON /BCOMP/ INWTEN(JELEM),INWTR (JELEM),INWTS (JELEM),
     1                INWTRV (JELEM),INWTDW(JELEM),
     2                INWORDP(JWORK),INBITP(JWORK)
C             INWTEN   -  WOKING TABLE
C             INWTR     -  WORKING TABLE REFERENCE
C             INWTS     -  WORKING SCALE
C             INWTRV    -  WORKING REFERENCE VALUE
C             INWTDW    -  WORKING DATA WIDTH

