C Copyright 1981-2016 ECMWF.
C
C This software is licensed under the terms of the Apache Licence 
C Version 2.0 which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
C
C In applying this licence, ECMWF does not waive the privileges and immunities 
C granted to it by virtue of its status as an intergovernmental organisation 
C nor does it submit to any jurisdiction.
C

      INTEGER FUNCTION GETTRU()
C
C---->
C*****GETTRU*
C
C     PURPOSE
C     -------
C
C      Finding Corresponding Truncation 
C
#include "parim.h"
#include "nofld.common"

      GETTRU = NORESO

        CALL INTLOG(JP_DEBUG,
     X 'GETTRU: Corresponding Truncation',GETTRU)
C
      RETURN
      END
