C Copyright 1981-2016 ECMWF.
C
C This software is licensed under the terms of the Apache Licence 
C Version 2.0 which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
C
C In applying this licence, ECMWF does not waive the privileges and immunities 
C granted to it by virtue of its status as an intergovernmental organisation 
C nor does it submit to any jurisdiction.
C
#ifndef INTISL_H
#define INTISL_H

#define MISSING (-999.0)
#define NJ      3
#define NPOINTS 23
#define NGAUSS  10
#define SEA     0
#define LAND    1
#define MASTERTHRESHOLD  0.5
#define OLDLSMTHRESHOLD  0.5
#define NWEST 1
#define NEAST 2
#define SWEST 3
#define SEAST 4
#define NORTH 1
#define SOUTH 2

#endif
