C Copyright 1981-2016 ECMWF.
C
C This software is licensed under the terms of the Apache Licence 
C Version 2.0 which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
C
C In applying this licence, ECMWF does not waive the privileges and immunities 
C granted to it by virtue of its status as an intergovernmental organisation 
C nor does it submit to any jurisdiction.
C

C
      COMMON /BCOMCT/ NREF(JCTAB)   ,NSTART(JCTAB) ,NLEN(JCTAB),
     1               NCODNUM(JCTST),NSTARTC(JCTST),
     2               NLENC(JCTST)
C
C             NREF      - TABLE C REFERENCE
C             NSTART    - STARTING POINTERS TO ARRAY NCODNUM
C             NLEN      - LENGTHS
C             NCODNUM   - CODE/FLAG TABLE NUMBER
C             NSTARTC   - STARTING POINTERS TO ARRAY CTEXT
C             NLENC     - LENGTHS
C

