/**
* Copyright 1981-2016 ECMWF.
*
* This software is licensed under the terms of the Apache Licence
* Version 2.0 which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
*
* In applying this licence, ECMWF does not waive the privileges and immunities
* granted to it by virtue of its status as an intergovernmental organisation
* nor does it submit to any jurisdiction.
*/
#ifndef FORTVALUES_H
#define FORTVALUES_H

#include "common/fortint.h"

#define BYTE  0xFF & (fortint)

void mvbytes(char *, fortint * , int );
void rtbytes(fortint *, char * , int );

#endif /* End of FORTVALUES_H */
